"use strict";
// import from npm
var lodash_1 = require('lodash');
var Promise = require('bluebird');
/**
 * map for script names against utility objects
 * @type {Map<string, ScriptState>}
 */
var loadedScripts = new Map();
/**
 *
 * @param scripts {Scripts} - An object with all the scripts required. Keys are script names, values are URLs.
 */
function cache(scripts) {
    var scriptTags = {};
    lodash_1.forEach(scripts, function (script, name) {
        lodash_1.assignIn(scriptTags, (_a = {},
            _a[name] = {
                tag: getScript(script, name),
                // TODO think of a way of doing this using 'bind' or 'call'?
                onLoad: onLoad(name),
            },
            _a
        ));
        var _a;
    });
    return scriptTags;
}
exports.cache = cache;
/**
 * Callback to be fired when each script has loaded.
 * @param name {string} - The name of the string that has just loaded.
 */
function onLoad(name) {
    return function (callback) {
        var stored = loadedScripts.get(name);
        if (stored) {
            stored.promise.then(function () {
                stored.wasRejected ? callback(stored.error) : callback(null, stored);
            });
        }
    };
}
/**
 * Get a script from a remote location.
 * @param name {string} - The name of the script to be retrieved.
 * @param url {string} - The URL/location of the script to be retrieved.
 */
function getScript(url, name) {
    if (!loadedScripts.has(name)) {
        var tag_1 = document.createElement('script');
        var promise = new Promise(function (resolve, reject) {
            var body = document.getElementsByTagName('body')[0];
            // make sure the script type is javascript
            // and that scripts are loaded in order using
            // the 'async' option
            lodash_1.assignIn(tag_1, {
                type: 'text/javascript',
                async: false,
            });
            var handleResult = function (state) {
                return function (event) {
                    var stored = loadedScripts.get(name);
                    if (state === 'loaded') {
                        stored.hasLoaded = true;
                        resolve(url);
                    }
                    else if (state === 'error') {
                        stored.wasRejected = true;
                        reject(event);
                    }
                };
            };
            lodash_1.assignIn(tag_1, {
                onload: handleResult('loaded'),
                onerror: handleResult('error'),
                onreadystatechange: function () {
                    handleResult(tag_1.readyState);
                },
            });
            // add load and error event listeners
            tag_1.addEventListener('load', tag_1.onload);
            tag_1.addEventListener('error', tag_1.onerror);
            lodash_1.assignIn(tag_1, { src: url });
            body.appendChild(tag_1);
            return tag_1;
        });
        var scriptObject = {
            hasLoaded: false,
            wasRejected: false,
            promise: promise,
            tag: tag_1,
        };
        loadedScripts.set(name, scriptObject);
    }
    return loadedScripts.get(name);
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = cache;
