"use strict";
// import from npm
var lodash_1 = require('lodash');
/**
 * map for link names against utility objects
 * @type {Map<string, LinkState>}
 */
var loadedLinks = new Map();
/**
 * Get a style or other linked resource from a remote location.
 * @param name {string} - The name of the resource to be retrieved.
 * @param url {string} - The URL/location of the resource to be retrieved.
 */
function getLink(url, name) {
    if (!loadedLinks.has(name)) {
        var link = document.createElement('link');
        var body = document.getElementsByTagName('body')[0];
        lodash_1.assignIn(link, {
            type: 'text/css',
            href: url,
            rel: 'stylesheet',
        });
        body.appendChild(link);
        var linkObject = {
            hasLoaded: false,
            wasRejected: false,
            link: link,
        };
        loadedLinks.set(name, linkObject);
    }
    return loadedLinks.get(name);
}
exports.getLink = getLink;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = getLink;
